/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
/*******************************************************************************
This TestSuite is used to execute the TestCase based on:
1. TestCase category; (e.g., Autonmation, Inventory,..)
2. Plugins; ( such as BF, BF V71, STAX, ... under category) 
3. TestMethod; (e.g., testInit, testGetJobDefinitionIds, ... under Plugin)

 ******************************************************************************/
package com.ibm.rational.test.lm.spi.driver.test;

import com.ibm.rational.test.lm.spi.IAutomationSPI;
import com.ibm.rational.test.lm.spi.IInventorySPI;
import com.ibm.rational.test.lm.spi.IVirtualizationSPI;
import com.ibm.rational.test.lm.spi.driver.test.util.ClassTestMethod;
import com.ibm.rational.test.lm.spi.driver.test.util.PRAutomation;
import com.ibm.rational.test.lm.spi.driver.test.util.PRInventory;
import com.ibm.rational.test.lm.spi.driver.test.util.PRVirtualization;


import junit.framework.Test;
import junit.framework.TestSuite;



public class SPITestSuite extends TestSuite {

    private static final String LEVEL1_TESTSUITE_RUNNER_NAME = "SUITE: com.ibm.rational.test.lm.spi.driver.test";
    private static final String LEVEL2_TESTSUITE_AUTOMATION = "Automation";
    private static final String LEVEL2_TESTSUITE_INVENTORY = "Inventory";
    private static final String LEVEL2_TESTSUITE_VIRTUALIZATION = "Virtualization";
    
	
	public static Test suite() {
		
		TestSuite suite = new TestSuite(LEVEL1_TESTSUITE_RUNNER_NAME);
		TestSuite suiteLevel2Category = null;
		
		
		SPITestSuite myAllTests_TestSuite = new SPITestSuite();
		
		
		//$JUnit-BEGIN$
		
		
		//add Automation TestCase...		
		suiteLevel2Category = myAllTests_TestSuite.addAutomation(LEVEL2_TESTSUITE_AUTOMATION);
    	suite.addTest(suiteLevel2Category);

		
		//add Inventory TestCase...
		suiteLevel2Category = myAllTests_TestSuite.addInventory(LEVEL2_TESTSUITE_INVENTORY);
    	suite.addTest(suiteLevel2Category);
		
    	
		//add Virtualization TestCase...
		suiteLevel2Category = myAllTests_TestSuite.addVirtualization(LEVEL2_TESTSUITE_VIRTUALIZATION);
    	suite.addTest(suiteLevel2Category);
    	
    	
		//$JUnit-END$
		return suite;
	}

	
	private TestSuite addAutomation(String sProviderName){
		
		TestSuite suiteLevel2Automation = new TestSuite(sProviderName);
			    
	    
		//initialize the automation plugins...
		PRAutomation mySuite_Plugins = new PRAutomation();
		mySuite_Plugins.init();
		mySuite_Plugins.init_full();
		
		
		//get the automation plugins...
		int iAutomation_Registered_and_Configured_Num = mySuite_Plugins.getRegisteredAndConfiguredNum();
	    IAutomationSPI[] automationProviders = (IAutomationSPI[]) mySuite_Plugins.getProviders();
	    int[] iAutomation_Registered_and_Configured_Index = mySuite_Plugins.getRegisteredAndConfiguredIndex();		
	    String[] automationProviderPluginIds = mySuite_Plugins.getProviderPluginIds();
	    String[] baseJobDefinitionUrls = mySuite_Plugins.getUrls();
		
		
	    //get the test method in TestCase of Automation...
	    Automation objAutomation = new Automation();
	    ClassTestMethod methodName= new ClassTestMethod((Object)objAutomation);
	    methodName.getClassDeclaredMethods();
	    String[] strMethodName = new String[methodName.getMethodNum()];
	    strMethodName = methodName.getMethodNames();
	    objAutomation = null;
	    
	    
	    //run the test method against every plugins of Automation...	    
        for (int i =0; i< iAutomation_Registered_and_Configured_Num; i++)
        {
        	int iProviderIdx = iAutomation_Registered_and_Configured_Index[i];
        	        	
    		TestSuite suiteLevel3AutomationPlugins = new TestSuite(automationProviderPluginIds[iProviderIdx]);        	
    		for (int j= 0; j < strMethodName.length; j++){
        		suiteLevel3AutomationPlugins.addTest(new Automation(
        				strMethodName[j],
            			automationProviders[iProviderIdx],
            			automationProviderPluginIds[iProviderIdx],
            			baseJobDefinitionUrls[iProviderIdx]));
    		}

    		
    		suiteLevel2Automation.addTest(suiteLevel3AutomationPlugins);             	
        }
		
        return suiteLevel2Automation;        
	}
	
	
	private TestSuite addInventory(String sProviderName){
		
		TestSuite suiteLevel2Inventory = new TestSuite(sProviderName);
			    
	    
		//initialize the inventory plugins...
	    PRInventory mySuite_Plugins = new PRInventory();
		mySuite_Plugins.init();
		mySuite_Plugins.init_full();
		
		//get the inventory plugins...
		int iInventory_Registered_and_Configured_Num = mySuite_Plugins.getRegisteredAndConfiguredNum();
	    IInventorySPI[] inventoryProviders = (IInventorySPI[]) mySuite_Plugins.getProviders();
	    int[] iInventory_Registered_and_Configured_Index = mySuite_Plugins.getRegisteredAndConfiguredIndex();
		
	    String[] inventoryProviderPluginIds = mySuite_Plugins.getProviderPluginIds();
	    String[] baseAssetUrls = mySuite_Plugins.getUrls();
			    
	    	    
	    //get the test method in TestCase of Inventory...
	    Inventory objInventory = new Inventory();
	    ClassTestMethod methodName= new ClassTestMethod((Object)objInventory);
	    methodName.getClassDeclaredMethods();
	    String[] strMethodName = new String[methodName.getMethodNum()];
	    strMethodName = methodName.getMethodNames();
	    objInventory = null;
	    
	    
	    //run the test method against every plugins of Inventory...	    
        for (int i =0; i< iInventory_Registered_and_Configured_Num; i++)
        {
        	int iProviderIdx = iInventory_Registered_and_Configured_Index[i];
        	        	
    		TestSuite suiteLevel3InventoryPlugins = new TestSuite(inventoryProviderPluginIds[iProviderIdx]);        	
    		for (int j= 0; j < strMethodName.length; j++){
        		suiteLevel3InventoryPlugins.addTest(new Inventory(
        				strMethodName[j],
            			inventoryProviders[iProviderIdx],
            			inventoryProviderPluginIds[iProviderIdx],
            			baseAssetUrls[iProviderIdx]));
    		}

    		
    		suiteLevel2Inventory.addTest(suiteLevel3InventoryPlugins);             	
        }
		
        return suiteLevel2Inventory;        
	}
	
	
	private TestSuite addVirtualization(String sProviderName){
		
		TestSuite suiteLevel2Virtualization = new TestSuite(sProviderName);

	    
		//initialize the virtualization plugins...
	    PRVirtualization mySuite_Plugins = new PRVirtualization();
		mySuite_Plugins.init();
		mySuite_Plugins.init_full();
		
		//get the virtualization plugins...
		int iVirtualization_Registered_and_Configured_Num = mySuite_Plugins.getRegisteredAndConfiguredNum();
	    IVirtualizationSPI[] virtualizationProviders = (IVirtualizationSPI[]) mySuite_Plugins.getProviders();
	    int[] iVirtualization_Registered_and_Configured_Index = mySuite_Plugins.getRegisteredAndConfiguredIndex();
	    String[] virtualizationProviderPluginIds = mySuite_Plugins.getProviderPluginIds();
			    
	    
	    //get the test method in TestCase of Virtualization...
	    Virtualization objVirtualization = new Virtualization();
	    ClassTestMethod methodName= new ClassTestMethod((Object)objVirtualization);
	    methodName.getClassDeclaredMethods();
	    String[] strMethodName = new String[methodName.getMethodNum()];
	    strMethodName = methodName.getMethodNames();
	    objVirtualization = null;
	
	    
	    //run the test method against every plugins of Virtualization...	    
        for (int i =0; i< iVirtualization_Registered_and_Configured_Num; i++)
        {
        	int iProviderIdx = iVirtualization_Registered_and_Configured_Index[i];
        	
    		TestSuite suiteLevel3VirtualizationPlugins = new TestSuite(virtualizationProviderPluginIds[iProviderIdx]);        	
    		for (int j= 0; j < strMethodName.length; j++){
        		suiteLevel3VirtualizationPlugins.addTest(new Virtualization(
        				strMethodName[j],
        				virtualizationProviders[iProviderIdx],
        				virtualizationProviderPluginIds[iProviderIdx]));
    		}

    		
    		suiteLevel2Virtualization.addTest(suiteLevel3VirtualizationPlugins);             	
        }
		
        return suiteLevel2Virtualization;        
	}
		
}
