/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
/*******************************************************************************
This abstract class define one abstract method which will be implemented by subclass
and all other method are concrete...

 ******************************************************************************/
package com.ibm.rational.test.lm.spi.driver.test.util;

import java.util.HashMap;

import com.ibm.rational.test.lm.spi.IRestSPI;
import com.ibm.rational.test.lm.spi.driver.test.util.ProviderXml.ProviderServerInfo;

public abstract class PluginReader {

	private String sCategory = "";
	
    private String spiHostname = "";
    private String spiPortNumber = "";
    
    private final String SINGLE_SLASH = "/";
	private final String COLON = ":";
    
    private HashMap<String, String> providerToConfig = new HashMap<String, String>();
    
    private int iRegistered_Num = 0;
    private String [] urls = null;
    private IRestSPI[] providers = null;
    private String[] providerPluginIds = null;
    	    
    //the automation plug-in in registry are not the same as those in integration_config.xml
    //iAutomation_Registered_and_Configured_Num: the automation plug-in in registry while also configed in integration_config.xml file...
    //iAutomation_Config_Index keep the sequence iRegistered_Num:
    //iAutomation_Config_Index will skip the numbers not existed in xml
    private int iRegistered_and_Configured_Num = 0;
    private int[] iRegistered_and_Configured_Index = null;
	
	private String sCategoryPartURL= "";
	
	public abstract boolean init();
	

	public boolean init_full(){
		
        boolean success = false;

//	providerToConfig will be defined by the concrete class as below:
//        providerToConfig = ProviderXml.getAutomationProviderDocs();
//	        providerToConfig = ProviderXml.getInventoryProviderDocs();
        
        success = providerToConfig.size() > 0;
        ProviderServerInfo serverInfo;
        
//    	inventory providers and pluginIds will be defined by the concrete class as below:
//        inventoryProviders = Registry.getRegistry().getInventoryProviders();
//        inventoryProviderPluginIds = Registry.getRegistry().getInventoryProviderPluginIds();

        
    	iRegistered_Num = providers.length;
    	urls = new String[iRegistered_Num];
    	iRegistered_and_Configured_Index = new int[iRegistered_Num];
    	
    	int [] iNot_Config_Index = new int[iRegistered_Num];
    	int iNot_Config_Num = 0;
        
    	
        System.out.println();
        System.out.println(sCategory + " plug-in: (Both Registered & w/ configuration info):");
    	
        
        for (int i =0; i< iRegistered_Num; i++)
        {
        	
        	String strServerInfo = providerToConfig.get(providerPluginIds[i]);
        	
        	if (strServerInfo != null)
        	{	        		
                serverInfo = ProviderXml.getProviderServerInfo(strServerInfo);
                spiHostname = serverInfo.hostname;
                spiPortNumber = serverInfo.port;
                urls[i] = SINGLE_SLASH + spiHostname + COLON + spiPortNumber + sCategoryPartURL;
		        
		        //keep the automation found in integration_config.xml ...
		        iRegistered_and_Configured_Index[iRegistered_and_Configured_Num] = i;
		        iRegistered_and_Configured_Num++;
       					        
                System.out.println("\t" + providerPluginIds[i]);
                
        	}else{
        		iNot_Config_Index[iNot_Config_Num] = i;
        		
        		iNot_Config_Num++;
        	}
        	
        }
        
        //registered but not configured...
        if (iNot_Config_Num > 0){
            System.out.println(sCategory + " plug-in: (Only Registered but without configuration info): ");
        	
	        for (int i = 0; i< iNot_Config_Num; i++){
	        	
                System.out.println("\t" + providerPluginIds[iNot_Config_Index[i]]);
	        	
	        }
        	
        }
        
        //to display those configured but not registered...
        boolean bAutomation_Configed_No_Registered_Displayed = false;
     
        for(String strAutomationPluginId : providerToConfig.keySet()) {
            boolean bFound = false;
        	for (int i =0; i< iRegistered_Num; i++)
            {
            	if (strAutomationPluginId.equalsIgnoreCase(providerPluginIds[i]))
            	{
            		bFound  = true;
            		break;            		
            	}
            }
        	
        	if (!bFound){
        		if (!bAutomation_Configed_No_Registered_Displayed){
        	        System.out.println(sCategory + " plug-in: (Only w/ configuration info but not Registered): ");
			        bAutomation_Configed_No_Registered_Displayed = true;
        		}
        		
            	System.out.println("\t" + strAutomationPluginId);
        	}
        }

        
        	
        System.out.println();
       
        return success;	        	        
		
	}

	
	public void setCategory(String asCategory){
		this.sCategory = asCategory;
	}
	
	public void setProviderToConfig(HashMap<String, String> aproviderToConfig){
		this.providerToConfig = aproviderToConfig;
	}
	
	public void setCategoryPartURL(String asCategoryPartURL){
		this.sCategoryPartURL = asCategoryPartURL;
	}
	
	
	public void setProviders(IRestSPI[] aproviders){
		this.providers = aproviders;
	}
	
	
	public void setProviderPluginIds(String[] aproviderPluginIds){
		this.providerPluginIds = aproviderPluginIds;		
	}	
	
	
    public int getRegisteredAndConfiguredNum(){
    	return iRegistered_and_Configured_Num;
    }

    
    public IRestSPI[] getProviders(){
    	return providers;
    }

    
    public int[] getRegisteredAndConfiguredIndex(){
    	return iRegistered_and_Configured_Index;
    }
    
    
    public String[] getProviderPluginIds(){
    	return providerPluginIds;
    }
    
    
    //this method will be called by automation and inventory, but not virtualization...
    public String[] getUrls(){
    	return urls;
    }	
}
