/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.rational.test.lm.spi.sample.inventory;

import java.math.BigInteger;
import java.net.URI;

import com.ibm.ccl.soa.deploy.core.CoreFactory;
import com.ibm.ccl.soa.deploy.core.HostingLink;
import com.ibm.ccl.soa.deploy.core.Topology;
import com.ibm.ccl.soa.deploy.os.OperatingSystem;
import com.ibm.ccl.soa.deploy.os.OperatingSystemUnit;
import com.ibm.ccl.soa.deploy.os.OsFactory;
import com.ibm.ccl.soa.deploy.server.Server;
import com.ibm.ccl.soa.deploy.server.ServerFactory;
import com.ibm.ccl.soa.deploy.server.ServerUnit;
import com.ibm.rational.test.lm.spi.IInventorySPI;
import com.ibm.rational.test.lm.spi.ITracer;
import com.ibm.rational.test.lm.spi.common.IRestAPIResponseCodes;
import com.ibm.rational.test.lm.spi.common.IRestResponse;
import com.ibm.rational.test.lm.spi.common.SPIURI;
import com.ibm.rational.test.lm.spi.common.URIResourceType;
import com.ibm.rational.test.lm.spi.sample.util.AdapterXml;
import com.ibm.rational.test.lm.spi.sample.util.RestResponse;
import com.ibm.rational.test.lm.spi.sample.util.TopologyUtils;

/**
 * Implementation of the com.ibm.rational.test.lm.spi.inventory SPI extension point
 */
public class InventorySPI implements IInventorySPI 
{
    public InventorySPI() 
    {
        
    }

    public IRestResponse delete(URI uri) 
    {
        RestResponse response = new RestResponse();
        response.setResponseCode(IRestAPIResponseCodes.SC_NOT_IMPLEMENTED);
        return response;
    }

    public IRestResponse get(URI uri) 
    {
        RestResponse response = new RestResponse();                    
        SPIURI spiuri = SPIURI.parse(uri);
        URIResourceType resourceType = spiuri.getResourceType();
        if (resourceType.equals(URIResourceType.asset))
        {            
            try
            {
                String id = spiuri.getResourceID();
                if (id == null || id.equals(""))
                {
                    response.setXMLResponse(getAllAssetIds());
                }
                else if (id.equals("*"))
                {
                    response.setXMLResponse(getAllAssets());
                }
                else
                {
                    response.setXMLResponse(getAsset(id));
                }
                response.setResponseCode(IRestAPIResponseCodes.SC_OK);
            }
            catch (Exception e)
            {
                response.setResponseCode(IRestAPIResponseCodes.SC_INTERNAL_SERVER_ERROR);
            }
        }
        else
        {
            response.setResponseCode(IRestAPIResponseCodes.SC_NOT_IMPLEMENTED);
        }
        return response;
    }

    public void init(String initProperties, ITracer tracer) throws Exception 
    {
        System.out.println("Initing Inventory Sample with hostname: " + AdapterXml.getHostname(initProperties));
        System.out.println("Initing Inventory Sample with port: " + AdapterXml.getPort(initProperties));
    }

    public IRestResponse post(URI uri, String XMLBody) 
    {
        RestResponse response = new RestResponse();
        response.setResponseCode(IRestAPIResponseCodes.SC_NOT_IMPLEMENTED);
        return response;
    }
    
    @SuppressWarnings("unchecked")
    private String getAllAssetIds()
    {
        Topology serverTopology = CoreFactory.eINSTANCE.createTopology();
        ServerUnit serverUnit = ServerFactory.eINSTANCE.createServerUnit();
        Server server = ServerFactory.eINSTANCE.createServer();
        server.setSystemId("1");
        serverUnit.getHostingOrAnyCapabilities().add(server);
        serverTopology.getUnits().add(serverUnit);
        return TopologyUtils.topologyToXml(serverTopology);
    }
    
    @SuppressWarnings("unchecked")
    private String getAllAssets()
    {        
        Topology serverTopology = CoreFactory.eINSTANCE.createTopology();
        ServerUnit serverUnit = ServerFactory.eINSTANCE.createServerUnit();
        OperatingSystemUnit osUnit = OsFactory.eINSTANCE.createOperatingSystemUnit();
        
        Server server = ServerFactory.eINSTANCE.createServer();
        server.setSystemId("1");
        server.setName("testmachine");
        server.setCpuArchitecture("x86");
        server.setMemorySize(new BigInteger("2048"));
        
        OperatingSystem os = OsFactory.eINSTANCE.createOperatingSystem();
        os.setName("Windows XP");
        os.setHostname("testmachine.ibm.com");
        
        osUnit.getHostingOrAnyCapabilities().add(os);
        serverUnit.getHostingOrAnyCapabilities().add(server);
        serverTopology.getUnits().add(serverUnit);
        serverTopology.getUnits().add(osUnit);
        
        HostingLink hostingLink = CoreFactory.eINSTANCE.createHostingLink();
        hostingLink.setHosted(osUnit);
        serverUnit.getHostingLinks().add(hostingLink);
        return TopologyUtils.topologyToXml(serverTopology);
    }
    
    private String getAsset(String assetId)
    {
        //TODO: implement more than one asset
        return getAllAssets();
    }
}
